@@ -1,67 +1,48 @@
+# Python Markdown
+
+# A Python implementation of John Gruber's Markdown.
+
+# Documentation: https://python-markdown.github.io/
+# GitHub: https://github.com/Python-Markdown/markdown/
+# PyPI: https://pypi.org/project/Markdown/
+
+# Started by Manfred Stienstra (http://www.dwerg.net/).
+# Maintained for a few years by Yuri Takhteyev (http://www.freewisdom.org).
+# Currently maintained by Waylan Limberg (https://github.com/waylan),
+# Dmitry Shachnev (https://github.com/mitya57) and Isaac Muse (https://github.com/facelessuser).
+
+# Copyright 2007-2023 The Python Markdown Project (v. 1.7 and later)
+# Copyright 2004, 2005, 2006 Yuri Takhteyev (v. 0.2-1.6b)
+# Copyright 2004 Manfred Stienstra (the original version)
+
+# License: BSD (see LICENSE.md for details).
+
 """
-Python Markdown
+In version 3.0, a new, more flexible inline processor was added, [`markdown.inlinepatterns.InlineProcessor`][].   The
+original inline patterns, which inherit from [`markdown.inlinepatterns.Pattern`][] or one of its children are still
+supported, though users are encouraged to migrate.
 
-A Python implementation of John Gruber's Markdown.
+The new `InlineProcessor` provides two major enhancements to `Patterns`:
 
-Documentation: https://python-markdown.github.io/
-GitHub: https://github.com/Python-Markdown/markdown/
-PyPI: https://pypi.org/project/Markdown/
+1. Inline Processors no longer need to match the entire block, so regular expressions no longer need to start with
+  `r'^(.*?)'` and end with `r'(.*?)%'`. This runs faster. The returned [`Match`][re.Match] object will only contain
+   what is explicitly matched in the pattern, and extension pattern groups now start with `m.group(1)`.
 
-Started by Manfred Stienstra (http://www.dwerg.net/).
-Maintained for a few years by Yuri Takhteyev (http://www.freewisdom.org).
-Currently maintained by Waylan Limberg (https://github.com/waylan),
-Dmitry Shachnev (https://github.com/mitya57) and Isaac Muse (https://github.com/facelessuser).
+2.  The `handleMatch` method now takes an additional input called `data`, which is the entire block under analysis,
+    not just what is matched with the specified pattern. The method now returns the element *and* the indexes relative
+    to `data` that the return element is replacing (usually `m.start(0)` and `m.end(0)`).  If the boundaries are
+    returned as `None`, it is assumed that the match did not take place, and nothing will be altered in `data`.
 
-Copyright 2007-2018 The Python Markdown Project (v. 1.7 and later)
-Copyright 2004, 2005, 2006 Yuri Takhteyev (v. 0.2-1.6b)
-Copyright 2004 Manfred Stienstra (the original version)
+    This allows handling of more complex constructs than regular expressions can handle, e.g., matching nested
+    brackets, and explicit control of the span "consumed" by the processor.
 
-License: BSD (see LICENSE.md for details).
-
-INLINE PATTERNS
-=============================================================================
-
-Inline patterns such as *emphasis* are handled by means of auxiliary
-objects, one per pattern.  Pattern objects must be instances of classes
-that extend markdown.Pattern.  Each pattern object uses a single regular
-expression and needs support the following methods:
-
-    pattern.getCompiledRegExp() # returns a regular expression
-
-    pattern.handleMatch(m) # takes a match object and returns
-                           # an ElementTree element or just plain text
-
-All of python markdown's built-in patterns subclass from Pattern,
-but you can add additional patterns that don't.
-
-Also note that all the regular expressions used by inline must
-capture the whole block.  For this reason, they all start with
-'^(.*)' and end with '(.*)!'.  In case with built-in expression
-Pattern takes care of adding the "^(.*)" and "(.*)!".
-
-Finally, the order in which regular expressions are applied is very
-important - e.g. if we first replace http://.../ links with <a> tags
-and _then_ try to replace inline html, we would end up with a mess.
-So, we apply the expressions in the following order:
-
-* escape and backticks have to go before everything else, so
-  that we can preempt any markdown patterns by escaping them.
-
-* then we handle auto-links (must be done before inline html)
-
-* then we handle inline HTML.  At this point we will simply
-  replace all inline HTML strings with a placeholder and add
-  the actual HTML to a hash.
-
-* then inline images (must be done before links)
-
-* then bracketed links, first regular then reference-style
-
-* finally we apply strong and emphasis
 """
 
+from __future__ import annotations
+
 from . import util
 from collections import namedtuple
+from typing import TYPE_CHECKING, Match, Any
 import re
 import xml.etree.ElementTree as etree
 try:  # pragma: no cover
@@ -69,9 +50,29 @@ try:  # pragma: no cover
 except ImportError:  # pragma: no cover
     import htmlentitydefs as entities
 
+if TYPE_CHECKING:  # pragma: no cover
+    from markdown import Markdown
 
-def build_inlinepatterns(md, **kwargs):
-    """ Build the default set of inline patterns for Markdown. """
+
+def build_inlinepatterns(md: Markdown, **kwargs: Any) -> util.Registry:
+    """
+    Build the default set of inline patterns for Markdown.
+
+    The order in which processors and/or patterns are applied is very important - e.g. if we first replace
+    `http://.../` links with `<a>` tags and _then_ try to replace inline HTML, we would end up with a mess. So, we
+    apply the expressions in the following order:
+
+    * backticks and escaped characters have to be handled before everything else so that we can preempt any markdown
+      patterns by escaping them;
+
+    * then we handle the various types of links (auto-links must be handled before inline HTML);
+
+    * then we handle inline HTML.  At this point we will simply replace all inline HTML strings with a placeholder
+      and add the actual HTML to a stash;
+
+    * finally we apply strong, emphasis, etc.
+
+    """
     inlinePatterns = util.Registry()
     inlinePatterns.register(BacktickInlineProcessor(BACKTICK_RE), 'backtick', 190)
     inlinePatterns.register(EscapeInlineProcessor(ESCAPE_RE, md), 'escape', 180)
@@ -98,81 +99,80 @@ def build_inlinepatterns(md, **kwargs):
     return inlinePatterns
 
 
-"""
-The actual regular expressions for patterns
------------------------------------------------------------------------------
-"""
+# The actual regular expressions for patterns
+# -----------------------------------------------------------------------------
 
 NOIMG = r'(?<!\!)'
+""" Match not an image. Partial regular expression which matches if not preceded by `!`. """
 
-# `e=f()` or ``e=f("`")``
 BACKTICK_RE = r'(?:(?<!\\)((?:\\{2})+)(?=`+)|(?<!\\)(`+)(.+?)(?<!`)\2(?!`))'
+""" Match backtick quoted string (`` `e=f()` `` or ``` ``e=f("`")`` ```). """
 
-# \<
 ESCAPE_RE = r'\\(.)'
+""" Match a backslash escaped character (`\\<` or `\\*`). """
 
-# *emphasis*
 EMPHASIS_RE = r'(\*)([^\*]+)\1'
+""" Match emphasis with an asterisk (`*emphasis*`). """
 
-# **strong**
 STRONG_RE = r'(\*{2})(.+?)\1'
+""" Match strong with an asterisk (`**strong**`). """
 
-# __smart__strong__
 SMART_STRONG_RE = r'(?<!\w)(_{2})(?!_)(.+?)(?<!_)\1(?!\w)'
+""" Match strong with underscore while ignoring middle word underscores (`__smart__strong__`). """
 
-# _smart_emphasis_
 SMART_EMPHASIS_RE = r'(?<!\w)(_)(?!_)(.+?)(?<!_)\1(?!\w)'
+""" Match emphasis with underscore while ignoring middle word underscores (`_smart_emphasis_`). """
 
-# __strong _em__
 SMART_STRONG_EM_RE = r'(?<!\w)(\_)\1(?!\1)(.+?)(?<!\w)\1(?!\1)(.+?)\1{3}(?!\w)'
+""" Match strong emphasis with underscores (`__strong _em__`). """
 
-# ***strongem*** or ***em*strong**
 EM_STRONG_RE = r'(\*)\1{2}(.+?)\1(.*?)\1{2}'
+""" Match emphasis strong with asterisk (`***strongem***` or `***em*strong**`). """
 
-# ___strongem___ or ___em_strong__
 EM_STRONG2_RE = r'(_)\1{2}(.+?)\1(.*?)\1{2}'
+""" Match emphasis strong with underscores (`___emstrong___` or `___em_strong__`). """
 
-# ***strong**em*
 STRONG_EM_RE = r'(\*)\1{2}(.+?)\1{2}(.*?)\1'
+""" Match strong emphasis with asterisk (`***strong**em*`). """
 
-# ___strong__em_
 STRONG_EM2_RE = r'(_)\1{2}(.+?)\1{2}(.*?)\1'
+""" Match strong emphasis with underscores (`___strong__em_`). """
 
-# **strong*em***
 STRONG_EM3_RE = r'(\*)\1(?!\1)([^*]+?)\1(?!\1)(.+?)\1{3}'
+""" Match strong emphasis with asterisk (`**strong*em***`). """
 
-# [text](url) or [text](<url>) or [text](url "title")
 LINK_RE = NOIMG + r'\['
+""" Match start of in-line link (`[text](url)` or `[text](<url>)` or `[text](url "title")`). """
 
-# ![alttxt](http://x.com/) or ![alttxt](<http://x.com/>)
 IMAGE_LINK_RE = r'\!\['
+""" Match start of in-line image link (`![alttxt](url)` or `![alttxt](<url>)`). """
 
-# [Google][3]
 REFERENCE_RE = LINK_RE
+""" Match start of reference link (`[Label][3]`). """
 
-# ![alt text][2]
 IMAGE_REFERENCE_RE = IMAGE_LINK_RE
+""" Match start of image reference (`![alt text][2]`). """
 
-# stand-alone * or _
-NOT_STRONG_RE = r'((^|\s)(\*|_)(\s|$))'
+NOT_STRONG_RE = r'((^|(?<=\s))(\*{1,3}|_{1,3})(?=\s|$))'
+""" Match a stand-alone `*` or `_`. """
 
-# <http://www.123.com>
 AUTOLINK_RE = r'<((?:[Ff]|[Hh][Tt])[Tt][Pp][Ss]?://[^<>]*)>'
+""" Match an automatic link (`<http://www.example.com>`). """
 
-# <me@example.com>
 AUTOMAIL_RE = r'<([^<> !]+@[^@<> ]+)>'
+""" Match an automatic email link (`<me@example.com>`). """
 
-# <...>
 HTML_RE = r'(<(\/?[a-zA-Z][^<>@ ]*( [^<>]*)?|!--(?:(?!<!--|-->).)*--)>)'
+""" Match an HTML tag (`<...>`). """
 
-# "&#38;" (decimal) or "&#x26;" (hex) or "&amp;" (named)
 ENTITY_RE = r'(&(?:\#[0-9]+|\#x[0-9a-fA-F]+|[a-zA-Z0-9]+);)'
+""" Match an HTML entity (`&#38;` (decimal) or `&#x26;` (hex) or `&amp;` (named)). """
 
-# two spaces at end of line
 LINE_BREAK_RE = r'  \n'
+""" Match two spaces at end of line. """
 
 
-def dequote(string):
+def dequote(string: str) -> str:
     """Remove quotes from around a string."""
     if ((string.startswith('"') and string.endswith('"')) or
        (string.startswith("'") and string.endswith("'"))):
@@ -185,24 +185,45 @@ class EmStrongItem(namedtuple('EmStrongItem', ['pattern', 'builder', 'tags'])):
     """Emphasis/strong pattern item."""
 
 
-"""
-The pattern classes
------------------------------------------------------------------------------
-"""
+# The pattern classes
+# -----------------------------------------------------------------------------
 
 
 class Pattern:  # pragma: no cover
-    """Base class that inline patterns subclass. """
+    """
+    Base class that inline patterns subclass.
+
+    Inline patterns are handled by means of `Pattern` subclasses, one per regular expression.
+    Each pattern object uses a single regular expression and must support the following methods:
+    [`getCompiledRegExp`][markdown.inlinepatterns.Pattern.getCompiledRegExp] and
+    [`handleMatch`][markdown.inlinepatterns.Pattern.handleMatch].
+
+    All the regular expressions used by `Pattern` subclasses must capture the whole block.  For this
+    reason, they all start with `^(.*)` and end with `(.*)!`.  When passing a regular expression on
+    class initialization, the `^(.*)` and `(.*)!` are added automatically and the regular expression
+    is pre-compiled.
+
+    It is strongly suggested that the newer style [`markdown.inlinepatterns.InlineProcessor`][] that
+    use a more efficient and flexible search approach be used instead. However, the older style
+    `Pattern` remains for backward compatibility with many existing third-party extensions.
+
+    """
 
     ANCESTOR_EXCLUDES = tuple()
+    """
+    A collection of elements which are undesirable ancestors. The processor will be skipped if it
+    would cause the content to be a descendant of one of the listed tag names.
+    """
 
-    def __init__(self, pattern, md=None):
+    def __init__(self, pattern: str, md: Markdown | None = None):
         """
         Create an instant of an inline pattern.
 
-        Keyword arguments:
+        Arguments:
+            pattern: A regular expression that matches a pattern.
+            md: An optional pointer to the instance of `markdown.Markdown` and is available as
+                `self.md` on the class instance.
 
-        * pattern: A regular expression that matches a pattern
 
         """
         self.pattern = pattern
@@ -211,33 +232,28 @@ class Pattern:  # pragma: no cover
 
         self.md = md
 
-    @property
-    @util.deprecated("Use 'md' instead.")
-    def markdown(self):
-        # TODO: remove this later
-        return self.md
-
-    def getCompiledRegExp(self):
+    def getCompiledRegExp(self) -> re.Pattern:
         """ Return a compiled regular expression. """
         return self.compiled_re
 
-    def handleMatch(self, m):
+    def handleMatch(self, m: Match) -> etree.Element:
         """Return a ElementTree element from the given match.
 
         Subclasses should override this method.
 
-        Keyword arguments:
+        Arguments:
+            m: A match object containing a match of the pattern.
 
-        * m: A re match object containing a match of the pattern.
+        Returns: An ElementTree Element object.
 
         """
         pass  # pragma: no cover
 
-    def type(self):
+    def type(self) -> str:
         """ Return class name, to define pattern type """
         return self.__class__.__name__
 
-    def unescape(self, text):
+    def unescape(self, text: str) -> str:
         """ Return unescaped text given text with an inline placeholder. """
         try:
             stash = self.md.treeprocessors['inline'].stashed_nodes
@@ -251,36 +267,38 @@ class Pattern:  # pragma: no cover
                 if isinstance(value, str):
                     return value
                 else:
-                    # An etree Element - return text content only
+                    # An `etree` Element - return text content only
                     return ''.join(value.itertext())
         return util.INLINE_PLACEHOLDER_RE.sub(get_stash, text)
 
 
 class InlineProcessor(Pattern):
     """
-    Base class that inline patterns subclass.
+    Base class that inline processors subclass.
 
     This is the newer style inline processor that uses a more
     efficient and flexible search approach.
+
     """
 
-    def __init__(self, pattern, md=None):
+    def __init__(self, pattern: str, md: Markdown | None = None):
         """
-        Create an instant of an inline pattern.
+        Create an instant of an inline processor.
 
-        Keyword arguments:
-
-        * pattern: A regular expression that matches a pattern
+        Arguments:
+            pattern: A regular expression that matches a pattern.
+            md: An optional pointer to the instance of `markdown.Markdown` and is available as
+                `self.md` on the class instance.
 
         """
         self.pattern = pattern
         self.compiled_re = re.compile(pattern, re.DOTALL | re.UNICODE)
 
-        # Api for Markdown to pass safe_mode into instance
+        # API for Markdown to pass `safe_mode` into instance
         self.safe_mode = False
         self.md = md
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element | str | None, int | None, int | None]:
         """Return a ElementTree element from the given match and the
         start and end index of the matched text.
 
@@ -289,37 +307,45 @@ class InlineProcessor(Pattern):
 
         Subclasses should override this method.
 
-        Keyword arguments:
-
-        * m: A re match object containing a match of the pattern.
-        * data: The buffer current under analysis
+        Arguments:
+            m: A re match object containing a match of the pattern.
+            data: The buffer currently under analysis.
 
         Returns:
-
-        * el: The ElementTree element, text or None.
-        * start: The start of the region that has been matched or None.
-        * end: The end of the region that has been matched or None.
+            el: The ElementTree element, text or None.
+            start: The start of the region that has been matched or None.
+            end: The end of the region that has been matched or None.
 
         """
         pass  # pragma: no cover
 
 
 class SimpleTextPattern(Pattern):  # pragma: no cover
-    """ Return a simple text of group(2) of a Pattern. """
-    def handleMatch(self, m):
+    """ Return a simple text of `group(2)` of a Pattern. """
+    def handleMatch(self, m: Match) -> str:
+        """ Return string content of `group(2)` of a matching pattern. """
         return m.group(2)
 
 
 class SimpleTextInlineProcessor(InlineProcessor):
-    """ Return a simple text of group(1) of a Pattern. """
-    def handleMatch(self, m, data):
+    """ Return a simple text of `group(1)` of a Pattern. """
+    def handleMatch(self, m: Match, data: str) -> tuple[str, int, int]:
+        """ Return string content of `group(1)` of a matching pattern. """
         return m.group(1), m.start(0), m.end(0)
 
 
 class EscapeInlineProcessor(InlineProcessor):
     """ Return an escaped character. """
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[str | None, int, int]:
+        """
+        If the character matched by `group(1)` of a pattern is in [`ESCAPED_CHARS`][markdown.Markdown.ESCAPED_CHARS]
+        then return the integer representing the character's Unicode code point (as returned by [`ord`][]) wrapped
+        in [`util.STX`][markdown.util.STX] and [`util.ETX`][markdown.util.ETX].
+
+        If the matched character is not in [`ESCAPED_CHARS`][markdown.Markdown.ESCAPED_CHARS], then return `None`.
+        """
+
         char = m.group(1)
         if char in self.md.ESCAPED_CHARS:
             return '{}{}{}'.format(util.STX, ord(char), util.ETX), m.start(0), m.end(0)
@@ -329,15 +355,28 @@ class EscapeInlineProcessor(InlineProcessor):
 
 class SimpleTagPattern(Pattern):  # pragma: no cover
     """
-    Return element of type `tag` with a text attribute of group(3)
+    Return element of type `tag` with a text attribute of `group(3)`
     of a Pattern.
 
     """
-    def __init__(self, pattern, tag):
+    def __init__(self, pattern: str, tag: str):
+        """
+        Create an instant of an simple tag pattern.
+
+        Arguments:
+            pattern: A regular expression that matches a pattern.
+            tag: Tag of element.
+
+        """
         Pattern.__init__(self, pattern)
         self.tag = tag
+        """ The tag of the rendered element. """
 
-    def handleMatch(self, m):
+    def handleMatch(self, m: Match) -> etree.Element:
+        """
+        Return [`Element`][xml.etree.ElementTree.Element] of type `tag` with the string in `group(3)` of a
+        matching pattern as the Element's text.
+        """
         el = etree.Element(self.tag)
         el.text = m.group(3)
         return el
@@ -345,15 +384,28 @@ class SimpleTagPattern(Pattern):  # pragma: no cover
 
 class SimpleTagInlineProcessor(InlineProcessor):
     """
-    Return element of type `tag` with a text attribute of group(2)
+    Return element of type `tag` with a text attribute of `group(2)`
     of a Pattern.
 
     """
-    def __init__(self, pattern, tag):
+    def __init__(self, pattern: str, tag: str):
+        """
+        Create an instant of an simple tag processor.
+
+        Arguments:
+            pattern: A regular expression that matches a pattern.
+            tag: Tag of element.
+
+        """
         InlineProcessor.__init__(self, pattern)
         self.tag = tag
+        """ The tag of the rendered element. """
 
-    def handleMatch(self, m, data):  # pragma: no cover
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element, int, int]:  # pragma: no cover
+        """
+        Return [`Element`][xml.etree.ElementTree.Element] of type `tag` with the string in `group(2)` of a
+        matching pattern as the Element's text.
+        """
         el = etree.Element(self.tag)
         el.text = m.group(2)
         return el, m.start(0), m.end(0)
@@ -361,24 +413,35 @@ class SimpleTagInlineProcessor(InlineProcessor):
 
 class SubstituteTagPattern(SimpleTagPattern):  # pragma: no cover
     """ Return an element of type `tag` with no children. """
-    def handleMatch(self, m):
+    def handleMatch(self, m: Match) -> etree.Element:
+        """ Return empty [`Element`][xml.etree.ElementTree.Element] of type `tag`. """
         return etree.Element(self.tag)
 
 
 class SubstituteTagInlineProcessor(SimpleTagInlineProcessor):
     """ Return an element of type `tag` with no children. """
-    def handleMatch(self, m, data):
+    def handleMatch(self,  m: Match, data: str) -> tuple[etree.Element, int, int]:
+        """ Return empty [`Element`][xml.etree.ElementTree.Element] of type `tag`. """
         return etree.Element(self.tag), m.start(0), m.end(0)
 
 
 class BacktickInlineProcessor(InlineProcessor):
-    """ Return a `<code>` element containing the matching text. """
+    """ Return a `<code>` element containing the escaped matching text. """
     def __init__(self, pattern):
         InlineProcessor.__init__(self, pattern)
         self.ESCAPED_BSLASH = '{}{}{}'.format(util.STX, ord('\\'), util.ETX)
         self.tag = 'code'
+        """ The tag of the rendered element. """
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element | str, int, int]:
+        """
+        If the match contains `group(3)` of a pattern, then return a `code`
+        [`Element`][xml.etree.ElementTree.Element] which contains HTML escaped text (with
+        [`code_escape`][markdown.util.code_escape]) as an [`AtomicString`][markdown.util.AtomicString].
+
+        If the match does not contain `group(3)` then return the text of `group(1)` backslash escaped.
+
+        """
         if m.group(3):
             el = etree.Element(self.tag)
             el.text = util.AtomicString(util.code_escape(m.group(3).strip()))
@@ -393,7 +456,12 @@ class DoubleTagPattern(SimpleTagPattern):  # pragma: no cover
     Useful for strong emphasis etc.
 
     """
-    def handleMatch(self, m):
+    def handleMatch(self, m: Match) -> etree.Element:
+        """
+        Return [`Element`][xml.etree.ElementTree.Element] in following format:
+        `<tag1><tag2>group(3)</tag2>group(4)</tag2>` where `group(4)` is optional.
+
+        """
         tag1, tag2 = self.tag.split(",")
         el1 = etree.Element(tag1)
         el2 = etree.SubElement(el1, tag2)
@@ -409,7 +477,12 @@ class DoubleTagInlineProcessor(SimpleTagInlineProcessor):
     Useful for strong emphasis etc.
 
     """
-    def handleMatch(self, m, data):  # pragma: no cover
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element, int, int]:  # pragma: no cover
+        """
+        Return [`Element`][xml.etree.ElementTree.Element] in following format:
+        `<tag1><tag2>group(2)</tag2>group(3)</tag2>` where `group(3)` is optional.
+
+        """
         tag1, tag2 = self.tag.split(",")
         el1 = etree.Element(tag1)
         el2 = etree.SubElement(el1, tag2)
@@ -421,8 +494,9 @@ class DoubleTagInlineProcessor(SimpleTagInlineProcessor):
 
 class HtmlInlineProcessor(InlineProcessor):
     """ Store raw inline html and return a placeholder. """
-    def handleMatch(self, m, data):
-        rawhtml = self.unescape(m.group(1))
+    def handleMatch(self, m: Match, data: str) -> tuple[str, int, int]:
+        """ Store the text of `group(1)` of a pattern and return a placeholder string. """
+        rawhtml = self.backslash_unescape(self.unescape(m.group(1)))
         place_holder = self.md.htmlStash.store(rawhtml)
         return place_holder, m.start(0), m.end(0)
 
@@ -444,6 +518,18 @@ class HtmlInlineProcessor(InlineProcessor):
 
         return util.INLINE_PLACEHOLDER_RE.sub(get_stash, text)
 
+    def backslash_unescape(self, text):
+        """ Return text with backslash escapes undone (backslashes are restored). """
+        try:
+            RE = self.md.treeprocessors['unescape'].RE
+        except KeyError:  # pragma: no cover
+            return text
+
+        def _unescape(m):
+            return chr(int(m.group(1)))
+
+        return RE.sub(_unescape, text)
+
 
 class AsteriskProcessor(InlineProcessor):
     """Emphasis processor for handling strong and em matches inside asterisks."""
@@ -455,6 +541,7 @@ class AsteriskProcessor(InlineProcessor):
         EmStrongItem(re.compile(STRONG_RE, re.DOTALL | re.UNICODE), 'single', 'strong'),
         EmStrongItem(re.compile(EMPHASIS_RE, re.DOTALL | re.UNICODE), 'single', 'em')
     ]
+    """ The various strong and emphasis patterns handled by this processor. """
 
     def build_single(self, m, tag, idx):
         """Return single tag."""
@@ -564,7 +651,7 @@ class AsteriskProcessor(InlineProcessor):
         else:
             return self.build_single(m, tags, index)
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element, int, int]:
         """Parse patterns."""
 
         el = None
@@ -591,6 +678,7 @@ class UnderscoreProcessor(AsteriskProcessor):
         EmStrongItem(re.compile(SMART_STRONG_RE, re.DOTALL | re.UNICODE), 'single', 'strong'),
         EmStrongItem(re.compile(SMART_EMPHASIS_RE, re.DOTALL | re.UNICODE), 'single', 'em')
     ]
+    """ The various strong and emphasis patterns handled by this processor. """
 
 
 class LinkInlineProcessor(InlineProcessor):
@@ -598,7 +686,8 @@ class LinkInlineProcessor(InlineProcessor):
     RE_LINK = re.compile(r'''\(\s*(?:(<[^<>]*>)\s*(?:('[^']*'|"[^"]*")\s*)?\))?''', re.DOTALL | re.UNICODE)
     RE_TITLE_CLEAN = re.compile(r'\s')
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element | None, int | None, int | None]:
+        """ Return an `a` [`Element`][xml.etree.ElementTree.Element] or `(None, None, None)`. """
         text, index, handled = self.getText(data, m.end(0))
 
         if not handled:
@@ -716,7 +805,7 @@ class LinkInlineProcessor(InlineProcessor):
                 if c != ' ':
                     last = c
 
-            # We have a scenario: [test](link"notitle)
+            # We have a scenario: `[test](link"notitle)`
             # When we enter a string, we stop tracking bracket resolution in the main counter,
             # but we do keep a backup counter up until we discover where we might resolve all brackets
             # if the title string fails to resolve.
@@ -755,9 +844,10 @@ class LinkInlineProcessor(InlineProcessor):
 
 
 class ImageInlineProcessor(LinkInlineProcessor):
-    """ Return a img element from the given match. """
+    """ Return a `img` element from the given match. """
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element | None, int | None, int | None]:
+        """ Return an `img` [`Element`][xml.etree.ElementTree.Element] or `(None, None, None)`. """
         text, index, handled = self.getText(data, m.end(0))
         if not handled:
             return None, None, None
@@ -783,7 +873,11 @@ class ReferenceInlineProcessor(LinkInlineProcessor):
 
     RE_LINK = re.compile(r'\s?\[([^\]]*)\]', re.DOTALL | re.UNICODE)
 
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element | None, int | None, int | None]:
+        """
+        Return [`Element`][xml.etree.ElementTree.Element] returned by `makeTag` method or `(None, None, None)`.
+
+        """
         text, index, handled = self.getText(data, m.end(0))
         if not handled:
             return None, None, None
@@ -792,7 +886,7 @@ class ReferenceInlineProcessor(LinkInlineProcessor):
         if not handled:
             return None, None, None
 
-        # Clean up linebreaks in id
+        # Clean up line breaks in id
         id = self.NEWLINE_CLEANUP_RE.sub(' ', id)
         if id not in self.md.references:  # ignore undefined refs
             return None, m.start(0), end
@@ -803,9 +897,9 @@ class ReferenceInlineProcessor(LinkInlineProcessor):
 
     def evalId(self, data, index, text):
         """
-        Evaluate the id portion of [ref][id].
+        Evaluate the id portion of `[ref][id]`.
 
-        If [ref][] use [ref].
+        If `[ref][]` use `[ref]`.
         """
         m = self.RE_LINK.match(data, pos=index)
         if not m:
@@ -817,7 +911,8 @@ class ReferenceInlineProcessor(LinkInlineProcessor):
                 id = text.lower()
         return id, end, True
 
-    def makeTag(self, href, title, text):
+    def makeTag(self, href: str, title: str, text: str) -> etree.Element:
+        """ Return an `a` [`Element`][xml.etree.ElementTree.Element]. """
         el = etree.Element('a')
 
         el.set('href', href)
@@ -829,16 +924,17 @@ class ReferenceInlineProcessor(LinkInlineProcessor):
 
 
 class ShortReferenceInlineProcessor(ReferenceInlineProcessor):
-    """Short form of reference: [google]. """
+    """Short form of reference: `[google]`. """
     def evalId(self, data, index, text):
-        """Evaluate the id from of [ref]  """
+        """Evaluate the id of `[ref]`.  """
 
         return text.lower(), index, True
 
 
 class ImageReferenceInlineProcessor(ReferenceInlineProcessor):
-    """ Match to a stored reference and return img element. """
-    def makeTag(self, href, title, text):
+    """ Match to a stored reference and return `img` element. """
+    def makeTag(self, href: str, title: str, text: str) -> etree.Element:
+        """ Return an `img` [`Element`][xml.etree.ElementTree.Element]. """
         el = etree.Element("img")
         el.set("src", href)
         if title:
@@ -848,16 +944,17 @@ class ImageReferenceInlineProcessor(ReferenceInlineProcessor):
 
 
 class ShortImageReferenceInlineProcessor(ImageReferenceInlineProcessor):
-    """ Short form of inage reference: ![ref]. """
+    """ Short form of image reference: `![ref]`. """
     def evalId(self, data, index, text):
-        """Evaluate the id from of [ref]  """
+        """Evaluate the id of `[ref]`.  """
 
         return text.lower(), index, True
 
 
 class AutolinkInlineProcessor(InlineProcessor):
-    """ Return a link Element given an autolink (`<http://example/com>`). """
-    def handleMatch(self, m, data):
+    """ Return a link Element given an auto-link (`<http://example/com>`). """
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element, int, int]:
+        """ Return an `a` [`Element`][xml.etree.ElementTree.Element] of `group(1)`. """
         el = etree.Element("a")
         el.set('href', self.unescape(m.group(1)))
         el.text = util.AtomicString(m.group(1))
@@ -866,9 +963,10 @@ class AutolinkInlineProcessor(InlineProcessor):
 
 class AutomailInlineProcessor(InlineProcessor):
     """
-    Return a mailto link Element given an automail link (`<foo@example.com>`).
+    Return a `mailto` link Element given an auto-mail link (`<foo@example.com>`).
     """
-    def handleMatch(self, m, data):
+    def handleMatch(self, m: Match, data: str) -> tuple[etree.Element, int, int]:
+        """ Return an [`Element`][xml.etree.ElementTree.Element] containing a `mailto` link  of `group(1)`. """
         el = etree.Element('a')
         email = self.unescape(m.group(1))
         if email.startswith("mailto:"):
